@@ -6,63 +6,68 @@
 import enum
 from typing import Dict, List, Tuple, Optional
 
-from pydantic import BaseModel, validator
-
-
-class Model(BaseModel):
-    class Config:
-        extra = "forbid"
+from pydantic import validator
+from .util import Model, _generating_documentation
 
 
 class ParamData(Model):
     """Various ways to modify parameters"""
 
-    # Rename parameter
+    #: Set parameter name to this
     name: Optional[str] = None
 
-    # C++ type emitted
+    #: Change C++ type emitted
     x_type: Optional[str] = None
 
-    # Default value for parameter
+    #: Default value for parameter
     default: Optional[str] = None
 
+    #: Force this to be an 'out' parameter
+    #:
+    #: .. seealso:: :ref:`autowrap_out_params`
+    #:
     force_out: bool = False
 
+    #: Force an array size
     array_size: Optional[int] = None
 
+    #: Ignore this parameter
     ignore: bool = False
 
 
 class BufferType(str, enum.Enum):
+
+    #: The buffer must indicate that it is readable (such as bytes, or bytearray)
     IN = "in"
+
+    #: The buffer must indicate that it is writeable (such as a bytearray)
     OUT = "out"
+
+    #: The buffer must indicate that it readable or writeable (such as a bytearray)
     INOUT = "inout"
 
 
 class BufferData(Model):
-    """Describes buffers"""
 
-    # Indicates what type of buffer is required: out/inout buffers require
-    # a writeable buffer such as a bytearray, but in only needs a readonly
-    # buffer (such as bytes)
+    #: Indicates what type of python buffer is required
     type: BufferType
 
-    # Name of source parameter -- user must pass in something that implements
-    # the buffer protocol (eg bytes, bytearray)
+    #: Name of C++ parameter that the buffer will use
     src: str
 
-    # Name of length parameter. An out-only parameter, it will be set to the size
-    # of the src buffer, and will be returned so the caller can determine how
-    # many bytes were written
+    #: Name of the C++ length parameter. An out-only parameter, it will be set
+    #: to the size of the python buffer, and will be returned so the caller can
+    #: determine how many bytes were written
     len: str
 
-    # If specified, the minimum size of the passed in buffer
+    #: If specified, the minimum size of the python buffer
     minsz: Optional[int] = None
 
 
 class ReturnValuePolicy(enum.Enum):
     """
-        see pybind11 documentation for this
+    See `pybind11 documentation <https://pybind11.readthedocs.io/en/stable/advanced/functions.html#return-value-policies>`_
+    for what each of these values mean.
     """
 
     TAKE_OWNERSHIP = "take_ownership"
@@ -75,37 +80,57 @@ class ReturnValuePolicy(enum.Enum):
 
 
 class FunctionData(Model):
-    # If True, don't wrap this
+    """
+    Customize the way the autogenerator binds a function.
+
+    .. code-block:: yaml
+
+       functions:
+         # for non-overloaded functions, just specify the name + customizations
+         name_of_non_overloaded_fn:
+           # add customizations for function here
+
+         # For overloaded functions, specify the name, but each overload
+         # separately
+         my_overloaded_fn:
+           overloads:
+             int, int:
+               # customizations for `my_overloaded_fn(int, int)`
+             int, int, int:
+               # customizations for `my_overloaded_fn(int, int, int)`
+    """
+
+    #: If True, don't wrap this
     ignore: bool = False
 
-    # If True, don't wrap this, but provide a pure virtual implementation
-    # -> in theory we could autodetect this, but sometimes ignore
-    #    is used for when CppHeaderParser totally blows it
+    #: If True, don't wrap this, but provide a pure virtual implementation
     ignore_pure: bool = False
 
-    # Generate this in an `#ifdef`
+    #: Generate this in an `#ifdef`
     ifdef: Optional[str] = None
-    # Generate this in an `#ifndef`
+    #: Generate this in an `#ifndef`
     ifndef: Optional[str] = None
 
-    # Use this code instead of the generated code
+    #: Use this code instead of the generated code
     cpp_code: Optional[str] = None
 
-    # Docstring for the function
+    #: Docstring for the function
     doc: Optional[str] = None
 
-    # If True, prepends an underscore to the python name
+    #: If True, prepends an underscore to the python name
     internal: bool = False
 
-    # Set this to rename the function
+    #: Use this to set the name of the function as exposed to python
     rename: Optional[str] = None
 
-    # Mechanism to override individual parameters
+    #: Mechanism to override individual parameters
     param_override: Dict[str, ParamData] = {}
 
     #: If specified, put the function in a sub.pack.age
     subpackage: Optional[str] = None
 
+    #: By default, robotpy-build will release the GIL whenever a wrapped
+    #: function is called.
     no_release_gil: Optional[bool] = None
 
     buffers: List[BufferData] = []
@@ -125,6 +150,30 @@ class FunctionData(Model):
     #: list is the template parameters for that function
     template_impls: Optional[List[List[str]]] = None
 
+    #: Specify a transformation lambda to be used when this virtual function
+    #: is called from C++. This inline code should be a lambda that has the same
+    #: arguments as the original C++ virtual function, except the first argument
+    #: will be a py::function with the python overload
+    #:
+    #: cpp_code should also be specified for this to be useful
+    #:
+    #: For example, to transform a function that takes an iostream into a function
+    #: that returns a string:
+    #:
+    #: .. code-block:: yaml
+    #:
+    #:    cpp_code: |
+    #:      [](MyClass* self) {
+    #:        return "string";
+    #:      }
+    #:    virtual_xform: |
+    #:      [](py::function fn, MyClass* self, std::iostream &is) {
+    #:         std::string d = py::cast(fn());
+    #:         is << d;
+    #:      }
+    #:
+    virtual_xform: Optional[str] = None
+
     @validator("overloads", pre=True)
     def validate_overloads(cls, value):
         for k, v in value.items():
@@ -133,14 +182,16 @@ class FunctionData(Model):
         return value
 
 
-FunctionData.update_forward_refs()
+if not _generating_documentation:
+    FunctionData.update_forward_refs()
 
 
 class PropAccess(enum.Enum):
 
     #: Determine read/read-write automatically:
-    #: - If a struct/union, default to readwrite
-    #: - If a class, default to readwrite if a basic type that isn't a
+    #:
+    #: * If a struct/union, default to readwrite
+    #: * If a class, default to readwrite if a basic type that isn't a
     #:   reference, otherwise default to readonly
     AUTOMATIC = "auto"
 
@@ -154,9 +205,11 @@ class PropAccess(enum.Enum):
 
 
 class PropData(Model):
+
+    #: If set to True, this property is not made available to python
     ignore: bool = False
 
-    #: Set the name of this property to the specified value
+    #: Set the python name of this property to the specified string
     rename: Optional[str]
 
     #: Python code access to this property
@@ -167,7 +220,11 @@ class PropData(Model):
 
 
 class EnumValue(Model):
+
+    #: If set to True, this property is not made available to python
     ignore: bool = False
+
+    #: Set the python name of this enum value to the specified string
     rename: Optional[str] = None
 
     #: Docstring for the enum value
@@ -176,11 +233,15 @@ class EnumValue(Model):
 
 class EnumData(Model):
 
-    # Docstring for the enum
+    #: Set your own docstring for the enum
     doc: Optional[str] = None
 
+    #: If set to True, this property is not made available to python
     ignore: bool = False
+
+    #: Set the python name of this enum to the specified string
     rename: Optional[str] = None
+
     value_prefix: Optional[str] = None
 
     #: If specified, put the enum in a sub.pack.age (ignored for
@@ -198,7 +259,7 @@ class ClassData(Model):
     ignore: bool = False
     ignored_bases: List[str] = []
 
-    # Specify fully qualified names for the bases
+    #: Specify fully qualified names for the bases
     base_qualnames: Dict[str, str] = {}
 
     attributes: Dict[str, PropData] = {}
@@ -236,7 +297,9 @@ class ClassData(Model):
     shared_ptr: bool = True
 
     #: If specified, put the class in a sub.pack.age. Ignored
-    #: for functions attached to a class.
+    #: for functions attached to a class. When template parameters
+    #: are used, must define subpackage on template instances
+    #: instead
     subpackage: Optional[str] = None
 
     #: Extra 'using' directives to insert into the trampoline and the
@@ -280,34 +343,129 @@ class ClassData(Model):
 
 
 class TemplateData(Model):
+    """
+    Instantiates a template as a python type. To customize the class,
+    add it to the ``classes`` key and specify the template type.
 
-    # Fully qualified name of template
+    Code to be wrapped:
+
+    .. code-block:: c++
+
+        template <typename T>
+        class MyClass {};
+
+    To bind ``MyClass<int>`` as the python class ``MyIntClass``, add this
+    to your YAML:
+
+    .. code-block:: yaml
+
+        classes:
+          MyClass:
+            template_params:
+            - T
+
+        templates:
+          MyIntClass:
+            qualname: MyClass
+            params:
+            - int
+    """
+
+    #: Fully qualified name of instantiated class
     qualname: str
 
-    # Template parameters to use
+    #: Template parameters to use
     params: List[str]
 
-    # TODO: other parameters useful for concrete types
+    #: If specified, put the template instantiation in a sub.pack.age
+    subpackage: Optional[str] = None
 
 
 class HooksDataYaml(Model):
     """
-        Format of the file in [tool.robotpy-build.wrappers."PACKAGENAME"]
-        generation_data
+    Format of the file in [tool.robotpy-build.wrappers."PACKAGENAME"]
+    generation_data
     """
 
     strip_prefixes: List[str] = []
+
+    #: Adds ``#include <FILENAME>`` directives to the top of the autogenerated
+    #: C++ file, after autodetected include dependencies are inserted.
     extra_includes: List[str] = []
-    # sticks these includes first, for really broken headers
+
+    #: Adds ``#include <FILENAME>`` directives after robotpy_build.h is
+    #: included, but before any autodetected include dependencies. Only use
+    #: this when dealing with broken headers.
     extra_includes_first: List[str] = []
+
+    #: Specify raw C++ code that will be inserted at the end of the
+    #: autogenerated file, inside a function. This is useful for extending
+    #: your classes or providing other customizations. The following C++
+    #: variables are available:
+    #:
+    #: * ``m`` is the ``py::module`` instance
+    #: * ``cls_CLASSNAME`` are ``py::class`` instances
+    #: * ... lots of other things too
+    #:
+    #: To see the full list, run a build and look at the generated code at
+    #: ``build/*/gensrc/**/*.cpp``
+    #:
+    #: Recommend that you use the YAML multiline syntax to specify it:
+    #:
+    #: .. code-block:: yaml
+    #:
+    #:    inline_code: |
+    #:      cls_CLASSNAME.def("get42", []() { return 42; });
     inline_code: Optional[str] = None
 
+    #: Key is the attribute (variable) name
+    #:
+    #: .. code-block:: yaml
+    #:
+    #:    attributes:
+    #:      my_variable:
+    #:        # customizations here, see PropData
+    #:
     attributes: Dict[str, PropData] = {}
+
+    #: Key is the class name
+    #:
+    #: .. code-block:: yaml
+    #:
+    #:    classes:
+    #:      CLASSNAME:
+    #:        # customizations here, see ClassData
+    #:
     classes: Dict[str, ClassData] = {}
+
+    #: Key is the function name
+    #:
+    #: .. code-block:: yaml
+    #:
+    #:    functions:
+    #:      fn_name:
+    #:        # customizations here, see FunctionData
+    #:
     functions: Dict[str, FunctionData] = {}
+
+    #: Key is the enum name, for enums at global scope
+    #:
+    #: .. code-block:: yaml
+    #:
+    #:    enums:
+    #:      MyEnum:
+    #:        # customizations here, see EnumData
+    #:
     enums: Dict[str, EnumData] = {}
 
-    #: Instantiates a template. Key is the name to give to the Python type
+    #: Instantiates a template. Key is the name to give to the Python type.
+    #:
+    #: .. code-block:: yaml
+    #:
+    #:    templates:
+    #:      ClassName:
+    #:        # customizations here, see TemplateData
+    #:
     templates: Dict[str, TemplateData] = {}
 
     @validator("attributes", pre=True)
